(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('rxjs'), require('rxjs/operators')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/forms', 'rxjs', 'rxjs/operators'], factory) :
	(factory((global.ng = global.ng || {}, global.ng.fileinputaccessor = {}),global.ng.core,global.ng.forms,global.Rx,global.Rx.Operators));
}(this, (function (exports,core,forms,rxjs,operators) { 'use strict';

var FileInputAccessor = (function () {
    function FileInputAccessor(_renderer, _elementRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this.onChange = function (_) { };
        this.onTouched = function () { };
        this.validator = this.generateAsyncValidator();
    }
    FileInputAccessor.prototype.ngOnChanges = function (changes) {
        if (changes.fileList && changes.fileList.currentValue.length === 0) {
            this._renderer.setProperty(this._elementRef.nativeElement, 'value', []);
        }
    };
    FileInputAccessor.prototype.writeValue = function (value) {
        var normalizedValue = value == null ? '' : value;
        this._renderer.setProperty(this._elementRef.nativeElement, 'value', normalizedValue);
    };
    FileInputAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = this.onChangeGenerator(fn);
    };
    FileInputAccessor.prototype.registerOnTouched = function (fn) { };
    FileInputAccessor.prototype.setDisabledState = function (isDisabled) {
        this._renderer.setProperty(this._elementRef.nativeElement, 'disabled', isDisabled);
    };
    FileInputAccessor.prototype.validate = function (c) {
        return this.validator(c);
    };
    FileInputAccessor.prototype.generateAsyncValidator = function () {
        var _this = this;
        return function (c) {
            if (!c.value || !c.value.length || c.disabled)
                return rxjs.Observable.of({});
            var errors = {};
            var loaders = [];
            var _loop_1 = function (f) {
                if (_this.size && _this.size < f.size) {
                    f.errors['fileSize'] = true;
                    errors['fileSize'] = true;
                }
                if (f.isImg && (_this.maxWidth || _this.maxHeight)) {
                    loaders.push(f.imgLoadReplay
                        .pipe(operators.take(1), operators.map(function (e) {
                        if (_this.maxWidth && f.imgWidth > _this.maxWidth) {
                            f.errors['imageWidth'] = true;
                            errors['imageWidth'] = true;
                        }
                        if (_this.maxHeight && f.imgHeight > _this.maxHeight) {
                            f.errors['imageHeight'] = true;
                            errors['imageHeight'] = true;
                        }
                        return e;
                    })));
                }
                if (!_this.allowedExt && !_this.allowedTypes)
                    return "continue";
                var extP = _this.generateRegExp(_this.allowedExt);
                var typeP = _this.generateRegExp(_this.allowedTypes);
                if (extP && !extP.test(f.name)) {
                    f.errors['fileExt'] = true;
                    errors['fileExt'] = true;
                }
                if (typeP && !typeP.test(f.type)) {
                    f.errors['fileType'] = true;
                    errors['fileType'] = true;
                }
            };
            for (var _i = 0, _a = c.value; _i < _a.length; _i++) {
                var f = _a[_i];
                _loop_1(f);
            }
            if (loaders.length) {
                return rxjs.Observable
                    .forkJoin.apply(rxjs.Observable, loaders).pipe(operators.map(function () {
                    return errors;
                }));
            }
            return rxjs.Observable.of(errors);
        };
    };
    
    FileInputAccessor.prototype.onChangeGenerator = function (fn) {
        var _this = this;
        return function (files) {
            var fileArr = [];
            if (!files || !files.length) {
                _this._renderer.setProperty(_this._elementRef.nativeElement, 'value', []);
                return fn(fileArr);
            }
            for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
                var f = files_1[_i];
                if (_this.withMeta && FileReader) {
                    var fr = new FileReader();
                    _this.generateFileMeta(f, fr);
                }
                f.errors = {};
                fileArr.push(f);
            }
            return fn(fileArr);
        };
    };
    
    FileInputAccessor.prototype.generateRegExp = function (pattern) {
        if (!pattern)
            return null;
        if (pattern instanceof RegExp) {
            return new RegExp(pattern);
        }
        else if (typeof pattern === 'string') {
            return new RegExp(pattern, 'ig');
        }
        else if (pattern instanceof Array) {
            return new RegExp("(" + pattern.join('|') + ")", 'ig');
        }
        return null;
    };
    FileInputAccessor.prototype.generateFileMeta = function (f, fr) {
        if (f.type.match(/text.*/)) {
            f.textLoadReplay = this.setText(f, fr);
        }
        else if (f.type.match(/image.*/)) {
            f.imgLoadReplay = this.setImage(f, fr);
        }
    };
    FileInputAccessor.prototype.setImage = function (f, fr) {
        f.isImg = true;
        var img = new Image();
        var imgLoadObs = rxjs.Observable
            .fromEvent(img, 'load')
            .pipe(operators.take(1), operators.map(function (e) {
            f.imgHeight = img.height;
            f.imgWidth = img.width;
            return e;
        }));
        var frLoadObs = rxjs.Observable
            .fromEvent(fr, 'load')
            .pipe(operators.take(1), operators.map(function (e) {
            f.imgSrc = fr.result;
            img.src = fr.result;
            return e;
        }));
        var onloadReplay = new rxjs.ReplaySubject(1);
        rxjs.Observable.forkJoin(imgLoadObs, frLoadObs).subscribe(onloadReplay);
        fr.readAsDataURL(f);
        return onloadReplay;
    };
    FileInputAccessor.prototype.setText = function (f, fr) {
        var onloadReplay = new rxjs.ReplaySubject(1);
        rxjs.Observable
            .fromEvent(fr, 'load')
            .pipe(operators.take(1), operators.map(function (e) {
            f.textContent = fr.result;
            return [e];
        }))
            .subscribe(onloadReplay);
        fr.readAsText(f);
        return onloadReplay;
    };
    return FileInputAccessor;
}());
FileInputAccessor.decorators = [
    { type: core.Directive, args: [{
                selector: 'input[type=file][formControl],input[type=file][formControlName],input[type=file][ngModel]',
                providers: [
                    {
                        provide: forms.NG_VALUE_ACCESSOR,
                        useExisting: core.forwardRef(function () { return FileInputAccessor; }),
                        multi: true
                    },
                    {
                        provide: forms.NG_ASYNC_VALIDATORS,
                        useExisting: core.forwardRef(function () { return FileInputAccessor; }),
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
FileInputAccessor.ctorParameters = function () { return [
    { type: core.Renderer2, },
    { type: core.ElementRef, },
]; };
FileInputAccessor.propDecorators = {
    'allowedExt': [{ type: core.Input, args: ['allowedExt',] },],
    'allowedTypes': [{ type: core.Input, args: ['allowedTypes',] },],
    'size': [{ type: core.Input, args: ['size',] },],
    'withMeta': [{ type: core.Input, args: ['withMeta',] },],
    'maxHeight': [{ type: core.Input, args: ['maxHeight',] },],
    'maxWidth': [{ type: core.Input, args: ['maxWidth',] },],
    'onChange': [{ type: core.HostListener, args: ['change', ['$event.target.files'],] },],
    'onTouched': [{ type: core.HostListener, args: ['blur',] },],
};

var FileInputAccessorModule = (function () {
    function FileInputAccessorModule() {
    }
    return FileInputAccessorModule;
}());
FileInputAccessorModule.decorators = [
    { type: core.NgModule, args: [{
                declarations: [FileInputAccessor],
                imports: [
                    forms.FormsModule,
                    forms.ReactiveFormsModule
                ],
                exports: [
                    FileInputAccessor,
                    forms.FormsModule,
                    forms.ReactiveFormsModule
                ]
            },] },
];
/** @nocollapse */
FileInputAccessorModule.ctorParameters = function () { return []; };

exports.FileInputAccessor = FileInputAccessor;
exports.FileInputAccessorModule = FileInputAccessorModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
